@@ -59,8 +59,8 @@ Emits warnings if not able to find a suitable executable."
   (let ((m2-home (ext:getenv "M2_HOME"))
         (m2 (ext:getenv "M2"))
         (mvn-executable (if (find :unix *features*)
-                               "mvn"
-                               "mvn.bat")))
+                            "mvn"
+                            "mvn.bat")))
     (when (and m2-home (probe-file m2-home))
       (let* ((m2-home (truename m2-home))
              (mvn-path (merge-pathnames 
@@ -88,14 +88,14 @@ Emits warnings if not able to find a suitable executable."
                 "where.exe"))
            (which-cmd-p 
             (handler-case 
-                (sys::run-program which-cmd nil)
+                (sys:run-program which-cmd nil)
               (t () nil))))
       (when which-cmd-p
         (dolist (mvn-path *mavens*)
           (let ((mvn 
                  (handler-case 
-                     (truename (read-line (sys::process-output 
-                                           (sys::run-program 
+                     (truename (read-line (sys:process-output 
+                                           (sys:run-program 
                                             which-cmd `(,mvn-path))))) 
                    (end-of-file () nil)
                    (t (e) 
@@ -115,32 +115,49 @@ Emits warnings if not able to find a suitable executable."
        (truename d)))
    (list (make-pathname :defaults (merge-pathnames "../lib/" (find-mvn))
                         :name nil :type nil)
-         #p"/usr/local/maven/lib/"))) ;; OpenBSD location suggested by Timo Myyrä
-                                  
+         ;; library location for homebrew maven package on OS X
+         (make-pathname :defaults (merge-pathnames "../libexec/lib/" (find-mvn))
+                        :name nil :type nil)
+         #p"/usr/local/share/java/maven3/lib/" ;; FreeBSD ports
+         #p"/usr/local/maven/lib/"))) ;; OpenBSD location suggested by Timo Myyr??
+
 (defparameter *mvn-libs-directory*
   nil
   "Location of 'maven-core-3.<m>.<p>.jar', 'maven-embedder-3.<m>.<p>.jar' etc.")
 
 (defun mvn-version ()
-  "Return the Maven version used by the Aether connector."
-  (let ((stream (sys::process-output
-                 (sys::run-program (truename (find-mvn)) '("-version"))))
-        (pattern (#"compile"
-                  'regex.Pattern
-                  "Apache Maven ([0-9]+)\\.([0-9]+)\\.([0-9]+)")))
-    (do ((line (read-line stream nil :eof) 
-              (read-line stream nil :eof)))
-        ((or (not line) (eq line :eof)) nil)
-      (let ((matcher (#"matcher" pattern line)))
-        (when (#"find" matcher)
-          (return-from mvn-version
-            (handler-case 
+  "Return the Maven version used by the Aether connector located by
+  FIND-MVN as a list of (MAJOR MINOR PATHLEVEL) integers.
+
+Signals a simple-error with additional information if this attempt fails."
+  (handler-case 
+      (let* ((process (sys:run-program (truename (find-mvn)) '("-version")))
+             (output (sys:process-output process))
+             (pattern (#"compile"
+                       'regex.Pattern
+                       "Apache Maven ([0-9]+)\\.([0-9]+)\\.([0-9]+)"))
+             lines)
+        (do ((line (read-line output nil :eof) 
+                   (read-line output nil :eof)))
+            ((or (not line) (eq line :eof)) nil)
+          (push line lines)
+          (let ((matcher (#"matcher" pattern line)))
+            (when (#"find" matcher)
+              (return-from mvn-version
                 (mapcar #'parse-integer 
                         `(,(#"group" matcher 1) 
                            ,(#"group" matcher 2) 
-                           ,(#"group" matcher 3)))
-              (t (e) 
-                (error "Failed to parse Maven version from ~A because~&~A." line e)))))))))
+                           ,(#"group" matcher 3)))))))
+        (when lines 
+          (signal "No parseable Maven version found in ~{~&  ~A~}" (nreverse lines)))
+        (let ((error (sys:process-error process)))
+          (do ((line (read-line error nil :eof) 
+                     (read-line error nil :eof)))
+              ((or (not line) (eq line :eof)) nil)
+            (push line lines)
+            (signal "Invocation of Maven returned the error ~{~&  ~A~}" (nreverse lines)))))
+    (t (e) 
+      (error "Failed to determine Maven version: ~A." e))))
 
 (defun ensure-mvn-version ()
   "Return t if Maven version is 3.0.3 or greater."
@@ -160,19 +177,22 @@ Emits warnings if not able to find a suitable executable."
 (defparameter *init* nil)
 
 (defun init (&optional &key (force nil))
- "Run the initialization strategy to bootstrap a Maven dependency node.
+  "Run the initialization strategy to bootstrap a Maven dependency node.
 
 Set *MVN-LIBS-DIRECTORY* to an explicit value before running this
 function in order to bypass the dynamic introspection of the location
 of the mvn executable with an explicit value."
- (unless (or force *mvn-libs-directory*)
-   (setf *mvn-libs-directory* (find-mvn-libs)))
+  (when force
+    (setf *session* nil
+          *repository-system* nil))
+  (unless (or force *mvn-libs-directory*)
+    (setf *mvn-libs-directory* (find-mvn-libs)))
   (unless (and *mvn-libs-directory*
                (probe-file *mvn-libs-directory*))
-   (error "Please obtain and install maven-3.0.4 locally from http://maven.apache.org/download.html, then set ABCL-ASDF:*MVN-DIRECTORY* appropiately."))
- (unless (ensure-mvn-version)
-   (error "We need maven-3.0.3 or later."))  (add-directory-jars-to-class-path *mvn-libs-directory* nil)
-  (setf *init* t))
+    (error "Please obtain and install maven-3.0.3 or later locally from <http://maven.apache.org/download.html>, then set ABCL-ASDF:*MVN-LIBS-DIRECTORY* to the directory containing maven-core-3.*.jar et. al."))
+  (unless (ensure-mvn-version)
+    (error "We need maven-3.0.3 or later."))  (add-directory-jars-to-class-path *mvn-libs-directory* nil)
+    (setf *init* t))
 
 (defun find-http-wagon ()
   "Find an implementation of the object that provides access to http and https resources.
@@ -196,10 +216,12 @@ hint."
   (java:jinterface-implementation 
    (#"getName" 
     (or
+     (ignore-errors  ;; Maven 3.2.5+
+       (jss:find-java-class 'aether.transport.wagon.WagonProvider))
      (ignore-errors  ;; Maven 3.1.0+
        (jss:find-java-class 'aether.connector.wagon.WagonProvider))
      (ignore-errors  ;; Maven 3.0.x
-      (jss:find-java-class 'org.sonatype.aether.connector.wagon.WagonProvider))))
+       (jss:find-java-class 'org.sonatype.aether.connector.wagon.WagonProvider))))
    "lookup"
    (lambda (role-hint)
      (cond 
@@ -209,7 +231,7 @@ hint."
         (progn 
           (format *maven-verbose* 
                   "~&WagonProvider stub passed '~A' as a hint it couldn't satisfy.~%" role-hint)
-           java:+null+))))
+          java:+null+))))
    "release"
    (lambda (wagon)
      (declare (ignore wagon)))))
@@ -219,7 +241,7 @@ hint."
    (ignore-errors 
      (#"newServiceLocator" 'org.apache.maven.repository.internal.MavenRepositorySystemUtils)) ;; maven-3.1.0
    (ignore-errors
-      (java:jnew "org.apache.maven.repository.internal.MavenServiceLocator")) ;; maven-3.0.4
+     (java:jnew "org.apache.maven.repository.internal.MavenServiceLocator")) ;; maven-3.0.4
    (ignore-errors
      (java:jnew "org.apache.maven.repository.internal.DefaultServiceLocator"))
    (ignore-errors  ;; maven-3.1.0 using org.eclipse.aether...
@@ -230,59 +252,77 @@ hint."
   (let ((locator 
          (find-service-locator))
         (wagon-provider-class 
-	 (or 
-	  (ignore-errors 
-	    (java:jclass "org.sonatype.aether.connector.wagon.WagonProvider"))
-	  (ignore-errors  ;; Maven-3.1.x 
-	    (jss:find-java-class 'aether.connector.wagon.WagonProvider))))
+         (or
+          (ignore-errors ;; Maven-3.3.x
+            (jss:find-java-class 'connector.transport.TransporterFactory))
+          (ignore-errors ;; Maven-3.2.5
+            (jss:find-java-class 'org.eclipse.aether.transport.wagon.WagonProvider))
+          (ignore-errors  ;; Maven-3.1.x 
+            (jss:find-java-class 'aether.connector.wagon.WagonProvider))
+          (ignore-errors 
+            (java:jclass "org.sonatype.aether.connector.wagon.WagonProvider"))))
         (wagon-repository-connector-factory-class
-	 (or 
-	  (ignore-errors 
-	    (java:jclass "org.sonatype.aether.connector.wagon.WagonRepositoryConnectorFactory"))
-	  (ignore-errors 
-	    (jss:find-java-class 'aether.connector.wagon.WagonRepositoryConnectorFactory))))
+         (or 
+          (ignore-errors 
+            (jss:find-java-class 'org.eclipse.aether.connector.basic.BasicRepositoryConnectorFactory))
+          (ignore-errors 
+            (jss:find-java-class 'aether.connector.wagon.WagonRepositoryConnectorFactory))
+          (ignore-errors 
+            (java:jclass "org.sonatype.aether.connector.wagon.WagonRepositoryConnectorFactory"))))
         (repository-connector-factory-class 
-	 (or 
-	  (ignore-errors 
-	    (java:jclass "org.sonatype.aether.spi.connector.RepositoryConnectorFactory"))
-	  (ignore-errors
-	    (jss:find-java-class 'aether.spi.connector.RepositoryConnectorFactory))))
+         (or
+          (ignore-errors
+            (jss:find-java-class 'org.eclipse.aether.spi.connector.RepositoryConnectorFactory))
+          (ignore-errors
+            (jss:find-java-class 'aether.spi.connector.RepositoryConnectorFactory))
+          (ignore-errors 
+            (java:jclass "org.sonatype.aether.spi.connector.RepositoryConnectorFactory"))))
         (repository-system-class
-	 (or
-	  (ignore-errors
-	    (java:jclass "org.sonatype.aether.RepositorySystem"))
-	  (ignore-errors 
-	    (jss:find-java-class 'aether.RepositorySystem)))))
-    (#"setServices" locator
-                    wagon-provider-class
-                   (java:jarray-from-list
-                    (list (make-wagon-provider))))
+         (or
+          (ignore-errors 
+            (jss:find-java-class 'org.eclipse.aether.RepositorySystem))
+          (ignore-errors 
+            (jss:find-java-class 'aether.RepositorySystem))
+          (ignore-errors
+            (java:jclass "org.sonatype.aether.RepositorySystem")))))
+
+    (if (equal wagon-provider-class (ignore-errors (jss:find-java-class 'TransporterFactory)))
+        ;;; Maven-3.3.3
+        (let ((wagon-transporter-factory (jss:new 'WagonTransporterFactory)))
+          (#"setWagonProvider" wagon-transporter-factory (make-wagon-provider))
+          (#"setServices" locator
+                          wagon-provider-class
+                          (java:jarray-from-list (list wagon-transporter-factory))))
+        (#"setServices" locator
+                        wagon-provider-class
+                        (java:jarray-from-list
+                         (list (make-wagon-provider)))))
     (#"addService" locator
                    repository-connector-factory-class
                    wagon-repository-connector-factory-class)
     (values (#"getService" locator
                            repository-system-class)
             locator)))
-        
+
 (defun make-session (repository-system)
   "Construct a new aether.RepositorySystemSession from the specified REPOSITORY-SYSTEM."
   (let ((session
-	 (or 
-	  (ignore-errors (#"newSession" 'org.apache.maven.repository.internal.MavenRepositorySystemUtils))
-	  (ignore-errors (java:jnew (jss:find-java-class "MavenRepositorySystemSession")))))
+         (or 
+          (ignore-errors (#"newSession" 'org.apache.maven.repository.internal.MavenRepositorySystemUtils))
+          (ignore-errors (java:jnew (jss:find-java-class "MavenRepositorySystemSession")))))
         (local-repository 
          (java:jnew (jss:find-java-class "LocalRepository")
-                  (namestring (merge-pathnames ".m2/repository/"
-                                               (user-homedir-pathname))))))
+                    (namestring (merge-pathnames ".m2/repository/"
+                                                 (user-homedir-pathname))))))
     (#"setLocalRepositoryManager" 
      session
      (or 
       (ignore-errors      ;; maven-3.1.0
-	(#"newLocalRepositoryManager" 
-	 repository-system session local-repository))
+        (#"newLocalRepositoryManager" 
+         repository-system session local-repository))
       (ignore-errors 
-	(#"newLocalRepositoryManager" 
-	 repository-system local-repository))))))
+        (#"newLocalRepositoryManager" 
+         repository-system local-repository))))))
 
 (defparameter *maven-http-proxy* nil
   "A string containing the URI of an http proxy for Maven to use.")
@@ -305,10 +345,10 @@ hint."
     (or 
      (ignore-errors
        (jss:new 'org.eclipse.aether.repository.Proxy
-		scheme host port authentication))
+                scheme host port authentication))
      (ignore-errors
        (jss:new 'org.sonatype.aether.repository.Proxy
-		scheme host port authentication)))))
+                scheme host port authentication)))))
 
 (defparameter *repository-system*  nil
   "The aether.RepositorySystem used by the Maeven Aether connector.")
@@ -333,15 +373,15 @@ If *MAVEN-HTTP-PROXY* is non-nil, parse its value as the http proxy."
                 proxy 
                 ;; A string specifying non proxy hosts, or null
                 java:+null+))))
-    *session*)
+  *session*)
 
 (defun make-artifact (artifact-string)
   "Return an instance of aether.artifact.DefaultArtifact initialized from ARTIFACT-STRING." 
-  (or 
+  (or
    (ignore-errors
-     (jss:new "org.sonatype.aether.util.artifact.DefaultArtifact" artifact-string))
+     (jss:new 'aether.artifact.DefaultArtifact artifact-string))
    (ignore-errors
-     (jss:new 'aether.artifact.DefaultArtifact artifact-string))))
+     (jss:new "org.sonatype.aether.util.artifact.DefaultArtifact" artifact-string))))
 
 (defun make-artifact-request () 
   "Construct a new aether.resolution.ArtifactRequest."
@@ -352,7 +392,7 @@ If *MAVEN-HTTP-PROXY* is non-nil, parse its value as the http proxy."
      (java:jnew "org.sonatype.aether.resolution.ArtifactRequest"))))
 
 ;;; TODO change this to work on artifact strings like log4j:log4j:jar:1.2.16
-(defun resolve-artifact (group-id artifact-id &optional (version "LATEST" versionp))
+(defun resolve-artifact (group-id artifact-id &key (version "LATEST" versionp))
   "Resolve artifact to location on the local filesystem.
 
 Declared dependencies are not attempted to be located.
@@ -364,11 +404,11 @@ Returns the Maven specific string for the artifact "
     (warn "Using LATEST for unspecified version."))
   (unless *init* (init))
   (let* ((artifact-string 
-	  (format nil "~A:~A:~A" group-id artifact-id version))
+          (format nil "~A:~A:~A" group-id artifact-id version))
          (artifact 
-	  (make-artifact artifact-string))
+          (make-artifact artifact-string))
          (artifact-request 
-	  (make-artifact-request)))
+          (make-artifact-request)))
     (#"setArtifact" artifact-request artifact)
     (#"addRepository" artifact-request (ensure-remote-repository))
     (#"toString" (#"getFile" 
@@ -383,13 +423,13 @@ Returns the Maven specific string for the artifact "
      (jss:new 'aether.repository.RemoteRepository id type url))))
 
 (defparameter *default-repository* 
-   "http://repo1.maven.org/maven2/")
+  "http://repo1.maven.org/maven2/")
 
 (defun add-repository (repository)
   (ensure-remote-repository :repository repository))
 
 (defparameter *maven-remote-repository*  nil
-    "The remote repository used by the Maven Aether embedder.")
+  "The remote repository used by the Maven Aether embedder.")
 (defun ensure-remote-repository (&key 
                                    (force nil)
                                    (repository *default-repository* repository-p))
@@ -405,11 +445,11 @@ Returns the Maven specific string for the artifact "
 
 
 (defun resolve-dependencies (group-id artifact-id 
-                             &optional  ;;; XXX Uggh.  Move to keywords when we get the moxie.
-                             (version "LATEST" versionp)
-                             (repository *maven-remote-repository* repository-p))
+                             &key
+                               (version "LATEST" versionp)
+                               (repository *maven-remote-repository* repository-p))
   "Dynamically resolve Maven dependencies for item with GROUP-ID and ARTIFACT-ID 
-optionally with a VERSION and a REPOSITORY.  Users of the function are advised 
+optionally with a VERSION and a REPOSITORY.  
 
 All recursive dependencies will be visited before resolution is successful.
 
@@ -421,25 +461,31 @@ in Java CLASSPATH representation."
   (unless versionp
     (warn "Using LATEST for unspecified version."))
   (let* ((coords 
-	  (format nil "~A:~A:~A" group-id artifact-id (if versionp version "LATEST")))
-	 (artifact 
-	  (make-artifact coords))
+          (format nil "~A:~A:~A" group-id artifact-id (if versionp version "LATEST")))
+         (artifact 
+          (make-artifact coords))
          (dependency 
           (java:jnew (jss:find-java-class 'aether.graph.Dependency)
-		     artifact (java:jfield (jss:find-java-class "JavaScopes") "RUNTIME")))
+                     artifact (java:jfield (jss:find-java-class "JavaScopes") "COMPILE")))
          (collect-request (java:jnew (jss:find-java-class "CollectRequest"))))
     (#"setRoot" collect-request dependency)
-    (#"addRepository" collect-request 
-                      (if repository-p
-                          (ensure-remote-repository :repository repository)
-                          (ensure-remote-repository)))
+     ;; Don't call addRepository if we explicitly specify a NIL repository
+    (unless (and repository-p (not repository))
+      (#"addRepository" collect-request 
+                        (if repository-p
+                            (ensure-remote-repository :repository repository)
+                            (ensure-remote-repository))))
     (let* ((node 
             (#"getRoot" (#"collectDependencies" (ensure-repository-system) (ensure-session) collect-request)))
-           (dependency-request 
+           (dependency-request
+            ;;; pre Maven-3.3.x
+            #+nil
             (java:jnew (jss:find-java-class "DependencyRequest")
-                       node java:+null+))
+                       node java:+null+)
+            (jss:new 'DependencyRequest))
            (nlg 
             (java:jnew (jss:find-java-class "PreorderNodeListGenerator"))))
+      (#"setRoot" dependency-request node)
       (#"resolveDependencies" (ensure-repository-system) (ensure-session) dependency-request)
       (#"accept" node nlg)
       (#"getClassPath" nlg))))
@@ -488,7 +534,7 @@ in Java CLASSPATH representation."
      "metadataResolving"
      #'log)))
 
-         
+
 (defmethod resolve ((string string))
   "Resolve a colon separated GROUP-ID:ARTIFACT-ID[:VERSION] reference to a Maven artifact.
 
@@ -502,13 +548,22 @@ artifact and all of its transitive dependencies."
     (cond 
       ((= (length result) 3)
        (resolve-dependencies 
-        (first result) (second result) (third result)))
+        (first result) (second result) :version (third result)))
       ((string= string "com.sun.jna:jna")
        (warn "Replacing request for no longer available com.sun.jna:jna with net.java.dev.jna:jna")
-       (resolve-dependencies "net.java.dev.jna" "jna" "LATEST"))
+       (resolve-dependencies "net.java.dev.jna" "jna" :version "LATEST"))
+      ((= (length result) 2)
+       (resolve-dependencies
+        (first result) (second result)))
       (t 
-       (setf result 
-             (apply #'resolve-dependencies (split-string string "/")))))))
-  
+       (destructuring-bind (group-id artifact-id &optional version repository)
+           (split-string string "/")
+         (setf result 
+               (apply #'resolve-dependencies group-id artifact-id
+                      (append (when version
+                                `(:version ,version))
+                              (when repository
+                                `(:repository ,repository))))))))))
+
 ;;; Currently the last file listed in ASDF
 (provide 'abcl-asdf)
